#define DMFILEDLL "dm_file.dll"

#define SEEK_CUR "C"
#define SEEK_END "E"
#define SEEK_SET "S"

binfile
	var/handle	= null		//the file handle string

	// Called when a binfile datum is deleted
	// If the file handle is still valid, close the file.
	Del()
		if(handle)
			close()


	proc

		// Open a file with the given filename and mode settings.
		// The default mode is "rb+", allowing you to read and write from an existing file in binary mode.
		// If the file does not exist, the proc fails
		// Returns 0 to signal an error condition (such as the file does not exist or is invalid)

		// note see C/C++ fopen() documentation for other mode settings

		open(filename, mode="rb+")

			handle = call(DMFILEDLL, "file_open")(filename, mode)

			if(handle=="-1")
				return 0
			return 1


		// Create a file.
		// Same as open(), but the default mode is "wb+", which allows reading and writing in binary mode.
		// If the file does not exist, it is created.
		// If the file already exists, it is truncated

		create(filename, mode="wb+")

			handle = call(DMFILEDLL, "file_open")(filename, mode)

			if(handle=="-1")
				return 0
			return 1


		// Close the file.
		// Should be called after all reads and writes are completed. This invalidates the binfile datum.
		// Should also be called before any standard DM file i/o is attempted on the same file.

		close()

			if( call(DMFILEDLL, "file_close")(handle) == "-1")
				return 0

			handle = null

			return 1

		// Flush the buffers associated with the file.
		// Should be called before switching between writing and reading data from the same file.

		flush()
			if( call(DMFILEDLL, "file_flush")(handle) == "-1")
				return null
			return 1

		// Returns true if the file position is at the end of the file

		eof()

			if( call(DMFILEDLL, "file_eof")(handle) == "0")
				return 0
			return 1


		// Get a string from the file and returns it
		// Reads until the first newline, the end of file, or until "max" characters has been read.
		// Returns null if an error occurred.

		getstring(var/max)

			var/result = call(DMFILEDLL, "file_gets")(handle, "[max]")

			if(result=="-1")
				return null

			return result


		// Write a string to the file
		// Returns false if an error occurred

		putstring(var/str)

			if( call(DMFILEDLL, "file_puts")(handle, str) == "-1")
				return 0

			return 1

		// Read a single byte from the file
		// Returns the byte, or null if an error occurred.

		getbyte()

			var/result = call(DMFILEDLL, "file_getbyte")(handle)

			if(result=="-1")
				return null

			return text2num(result)

		// Write a single byte to the file
		// Return false if an error occurred.

		putbyte(var/byte)

			var/result = call(DMFILEDLL, "file_putbyte")(handle, "[byte]")

			if(result=="-1")
				return 0

			return 1


		// Read a two-byte word from the file
		// Returns the word value, or null if error.
		// Set the optional parameter to non-zero to read in big-endian order (high-byte, low-byte)
		// Otherwise the default is little-endian order (low-byte, high-byte)

		getword(var/bigendian=0)

			var/result
			if(bigendian)
				result = call(DMFILEDLL, "file_getword")(handle, "B")
			else
				result = call(DMFILEDLL, "file_getword")(handle, "L")

			if(result == "-1")
				return null

			return text2num(result)


		// Write a two-byte word to the file
		// Returns false if an error occurred.
		// Set the optional parameter to non-zero to write in big-endian order (high-byte, low-byte)
		// Otherwise the default is little-endian order (low-byte, high-byte)

		putword(var/word, var/bigendian=0)

			var/result
			if(bigendian)
				result = call(DMFILEDLL, "file_putword")(handle, "[word]", "B")
			else
				result = call(DMFILEDLL, "file_putword")(handle, "[word]", "L")

			if(result=="-1")
				return 0

			return 1


		// Return the current file position, usually the byte offset from the start of the file
		// Returns null if an error occurred

		tell()
			var/pos = call(DMFILEDLL, "file_tell")(handle)


			if(pos == "-1")
				return null

			return text2num(pos)


		// Set the current file position
		// "origin" is one of:
		// SEEK_CUR (default) - "pos" is the change from current file position
		// SEEK_END - "pos" is from end of file
		// SEEK_SET - "posn" is from start of file

		seek(var/pos, var/origin=SEEK_CUR)

			var/result = call(DMFILEDLL, "file_seek")(handle, "[pos]", origin)

			if(result == "-1")
				return null

			return 1


		// Returns the last error code
		// 0 = no error
		// 1 = DLL function called with wrong number of paramters
		// 2 = failed to open file
		// 3 = unknown file handle
		// 4 = failed to close file
		// 5 = end of file reached
		// 6 = invalid file seek/tell
		// 7 = invalid seek origin

		error()
			var/result = call(DMFILEDLL, "last_error")()

			return text2num(result)


		// Returns a string describing the last error code

		error_string()
			var/result = call(DMFILEDLL, "last_error_str")()

			return result

		// Return the DLL version string

		version()
			return call(DMFILEDLL, "version")()

